﻿using System;
using System.Web.Http;
using Microsoft.ServiceBus;

namespace BranchAzureServices.Controllers
{
    public class BranchMessagingController : ApiController
    {
        private const string SbNamespace = "BaAzureHybrid";
        private const string UpdatesFromBranchesTopic = "UpdatesFromBranches";
        private const string UpdatesFromErpTopic = "UpdatesFromErp";

        private const string NamespaceConnectionString =
            "<Your Namespace Connection String>";

        private readonly NamespaceManager _manager =
            NamespaceManager.CreateFromConnectionString(NamespaceConnectionString);

        private const string PublishPolicy = "PublishMessages";
        private const string PublishPolicyKey = "<Your PublishPolidyKey>";
        private const string SubscriptionPolicy = "SubscribeMessages";
        private const string SubscriptionPolicyKey = "<Your SubscriptionPolidyKey>";

        private static readonly TimeSpan SasTimeToLive = TimeSpan.FromMinutes(10);

        public string Get(string topic, string subscription)
        {
            if (topic != UpdatesFromErpTopic
                ||
                !subscription.StartsWith("Branch", StringComparison.OrdinalIgnoreCase))
                return "Invalid input";

            if (!_manager.SubscriptionExists(topic, subscription))
                _manager.CreateSubscriptionAsync(topic, subscription).Wait();

            var sasUri = ServiceBusEnvironment.CreateServiceUri("https", SbNamespace, topic).ToString().Trim('/');

            return
                SharedAccessSignatureTokenProvider
                    .GetSharedAccessSignature
                    (
                        SubscriptionPolicy,
                        SubscriptionPolicyKey,
                        sasUri,
                        SasTimeToLive);
        }

        public string Get(string topic)
        {
            if (topic != UpdatesFromBranchesTopic)
                return "Invalid input";

            var sasUri = ServiceBusEnvironment.CreateServiceUri("https", SbNamespace, topic).ToString().Trim('/');

            return
                SharedAccessSignatureTokenProvider
                    .GetSharedAccessSignature
                    (
                        PublishPolicy,
                        PublishPolicyKey,
                        sasUri,
                        SasTimeToLive);
        }

        public void Delete(string topic, string subscription)
        {
            if (topic != UpdatesFromErpTopic
                ||
                !subscription.StartsWith("Branch", StringComparison.OrdinalIgnoreCase))
                return;

            if (_manager.SubscriptionExists(topic, subscription))
                _manager.DeleteSubscriptionAsync(topic, subscription).Wait();
        }

    }
}
