﻿using System;
using System.Linq;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;

using AzureHybrid.Models;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Auth;
using Microsoft.WindowsAzure.Storage.Table;

namespace SalesAnalytics
{
    class Program
    {
        private const string SbNamespace = "BaAzureHybrid";
        private const string UpdatesFromBranchesTopic = "UpdatesFromBranches";
        private const string DataAnalyticsSubscription = "DataAnalyticsSubscription";

        private const string SubscriptionPolicy = "SubscribeMessages";
        private const string SubscriptionPolicyKey = "<Your SubscriptionPolidyKey>";

        private const string StorageAccountName = "baazurehybridstorage";
        private const string StorageAccountKey = "mrT6T+oPQlXZJ31bLf7ifN+2k94MVXIWAcDruXqn2FKHi7z1n6/HY9PckedzWLL8KGeX8pLxUaB4mOq0QaUl/g==";
        private const string SalesTableName = "Sales";

        private static readonly string ServiceUri =
            ServiceBusEnvironment.CreateServiceUri("sb", SbNamespace, "").ToString();

        static void Main()
        {
            Console.WriteLine("SalesAnalytics WebJob Started...");


            var factorySettings = new MessagingFactorySettings
            {
                TransportType = TransportType.NetMessaging,
                TokenProvider =
                    TokenProvider.CreateSharedAccessSignatureTokenProvider(SubscriptionPolicy, SubscriptionPolicyKey)
            };

            var factory = MessagingFactory.Create(ServiceUri, factorySettings);
            var subscriptionClient = factory.CreateSubscriptionClient(UpdatesFromBranchesTopic, DataAnalyticsSubscription);

            const int maxMessages = 100;
            var messageTimeOut = TimeSpan.FromSeconds(5);

            var messages =
                subscriptionClient
                    .ReceiveBatchAsync(maxMessages, messageTimeOut)
                    .Result
                    .ToList();

            if (messages.Count == 0) 
                return;

            var credentials = new StorageCredentials(StorageAccountName, StorageAccountKey);
            var storageAccount = new CloudStorageAccount(credentials, true);
            var tableClient = storageAccount.CreateCloudTableClient();
            var table = tableClient.GetTableReference(SalesTableName);
            table.CreateIfNotExists();

            messages
                .ForEach(brokeredMessage =>
                {
                    var sale = brokeredMessage.GetBody<Sale>();
                    Console.WriteLine("SalesAnalytics WebJob received Sale for Customer {0}", sale.CustomerId);

                    var tableEntity = TableEntity.CreateTableEntity(
                        sale,
                        s => s.Product,
                        s => s.CustomerId.ToString()
                        );

                    var insertSingle = TableOperation.InsertOrMerge(tableEntity);
                    table.Execute(insertSingle);

                    brokeredMessage.CompleteAsync().Wait();
                });
        }
    }
}
