﻿<# Copyright 2015 Richard T. Broida

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 #>

cls
$ErrorActionPreference = "Stop"

$Namespace = "BaAzureHybrid"
$Location = "East US"

try
{
    Write-Output "Adding the [AzureSetupHelper.dll] assembly to the script..."
    $scriptPath = Split-Path (Get-Variable MyInvocation -Scope 0).Value.MyCommand.Path
    $packagesFolder = (Split-Path $scriptPath -Parent) + "\SetupHelper"
    $assembly = Get-ChildItem $packagesFolder -Include "AzureSetupHelper.dll" -Recurse
    Add-Type -Path $assembly.FullName

    Write-Output "The [AzureSetupHelper.dll] assembly has been successfully added to the script."    
}
catch [System.Exception]
{
    Write-Error("Could not add the AzureSetupHelper.dll assembly to the script. Make sure you build the solution before running the provisioning script.")
}

$CurrentNamespace = Get-AzureSBNamespace -Name $Namespace
if ($CurrentNamespace)
{
    Write-Output "The namespace [$Namespace] already exists."
}
else
{
    Write-Output "Creating the [$Namespace] namespace in the [$Location] region..."
    New-AzureSBNamespace -Name $Namespace -Location $Location -CreateACSNamespace $false -NamespaceType Messaging
    Write-Host "The [$Namespace] namespace in the [$Location] region has been successfully created."
}

Write-Output "Getting connection string for the [$Namespace] namespace..."
$sbr = Get-AzureSBAuthorizationRule -Namespace $Namespace
$ConnectionString = $sbr.ConnectionString

Write-Output "Creating a Setup object for the [$Namespace] namespace..."
$Setup = New-Object -TypeName AzureSetupHelper.Setup
Write-Output "Setup object for the [$Namespace] namespace has been successfully created."

$SetupResult = $Setup.Create($ConnectionString)
if ($SetupResult -eq "Success")
{
	Write-Output "Setup object has successfully created objects in the [$Namespace] namespace."
}
else
{
    Write-Error ("Setup object threw exception: " + $SetupResult)
}





