﻿using System;
using System.Collections.Generic;
using System.Threading;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;

namespace AzureSetupHelper
{
    public class Setup
    {
        private const string UpdatesFromBranchesTopic = "UpdatesFromBranches";
        private const string ErpSubscription = "ErpSubscription";
        private const string DataAnalyticsSubscription = "DataAnalyticsSubscription";

        private const string UpdatesFromErpTopic = "UpdatesFromErp";

        private const string PublishPolicyName = "PublishMessages";
        private const string SubscriptionPolicyName = "SubscribeMessages";
        private const string PublishPolicyKey = "<Your PublishPolidyKey>";
        private const string SubscriptionPolicyKey = "<Your SubscriptionPolidyKey>";

        private static readonly List<AccessRights> SendRights = new List<AccessRights> { AccessRights.Send };
        private static readonly List<AccessRights> ListenRights = new List<AccessRights> { AccessRights.Listen };

        private static readonly SharedAccessAuthorizationRule PublishPolicy = new SharedAccessAuthorizationRule(PublishPolicyName, SendRights)
        {
            ClaimType = "SharedAccessKey",
            PrimaryKey = PublishPolicyKey,
        };

        private static readonly SharedAccessAuthorizationRule SubscriptionPolicy = new SharedAccessAuthorizationRule(SubscriptionPolicyName,
            ListenRights)
        {
            ClaimType = "SharedAccessKey",
            PrimaryKey = SubscriptionPolicyKey,
        };

        private static void CreateTopic(NamespaceManager manager, string topicName)
        {
            if (manager.TopicExists(topicName)) return;

            var topicDescription = new TopicDescription(topicName);
            topicDescription.Authorization.Add(PublishPolicy);
            topicDescription.Authorization.Add(SubscriptionPolicy);

            manager.CreateTopicAsync(topicDescription).Wait();
        }

        private static void CreateSubscription(NamespaceManager manager, string topicName, string subscriptionName, SqlFilter filter = null)
        {
            if (manager.SubscriptionExists(topicName, subscriptionName)) return;

            if (filter == null)
                manager.CreateSubscriptionAsync(topicName, subscriptionName).Wait();
            else
                manager.CreateSubscriptionAsync(topicName, subscriptionName, filter).Wait();
        }

        public string Create(string connectionString)
        {
            var attempts = 0;
            var manager = NamespaceManager.CreateFromConnectionString(connectionString);
            
            while (true)
            {
                try
                {
                    CreateTopic(manager, UpdatesFromBranchesTopic);
                    CreateSubscription(manager, UpdatesFromBranchesTopic, ErpSubscription);
                    CreateSubscription(manager, UpdatesFromBranchesTopic, DataAnalyticsSubscription);
                    
                    CreateTopic(manager, UpdatesFromErpTopic);

                    return "Success";
                }
                catch (Exception ex)
                {
                    if (attempts < 5)
                    {
                        Thread.Sleep(5000);
                        ++attempts;
                    }
                    else
                    {
                        return ex.ToString();
                    }
                }
            }
        }
    }
}
