﻿using System;
using System.Net.Http;
using AzureHybrid.Models;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;

namespace BranchPublisher
{
    internal class Program
    {
        private const string SbNamespace = "BaAzureHybrid";
        private const string UpdatesFromBranchesTopic = "UpdatesFromBranches";
        
        private static readonly string ServiceUri =
            ServiceBusEnvironment.CreateServiceUri("sb", SbNamespace, "").ToString();

        private static Sale CreateTestSale()
        {
            return new Sale
            {
                Product = "Blue Azure Propane Tank"
            };
        }

        private static void Main()
        {
            Console.WriteLine("Starting the BranchPublisher...");

            var tokenUri = string.Format("api/BranchMessaging/{0}", UpdatesFromBranchesTopic);

            using (var httpClient = new HttpClient { BaseAddress = new Uri("http://branchazureservices.azurewebsites.net/") })
            {

                var getTokenResponse = httpClient.GetAsync(tokenUri).Result;
                getTokenResponse.EnsureSuccessStatusCode();
                var subscriptionToken =
                    getTokenResponse
                        .Content
                        .ReadAsStringAsync()
                        .Result
                        .Replace("\"", "");

                var factorySettings = new MessagingFactorySettings
                {
                    TransportType = TransportType.NetMessaging,
                    TokenProvider =
                        TokenProvider.CreateSharedAccessSignatureTokenProvider(subscriptionToken)
                };

                var factory = MessagingFactory.Create(ServiceUri, factorySettings);
                var publisherClient = factory.CreateTopicClient(UpdatesFromBranchesTopic);

                var quitChar = ' '; 
                Console.WriteLine("Press <enter> to publish and q to quit...");

                while (quitChar != 'q')
                {
                    var sale = CreateTestSale();
                    var message = new BrokeredMessage(sale);

                    publisherClient.SendAsync(message).Wait();

                    Console.WriteLine("Published Sale of {0} for Customer {1}", sale.Quantity, sale.CustomerId);

                    quitChar = Console.ReadKey().KeyChar;
                }

            }

        }
    }
}
