﻿using System;
using System.Net.Http;
using AzureHybrid.Models;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;

namespace ErpSubscriber
{
    internal class Program
    {
        private const string SbNamespace = "BaAzureHybrid";
        private const string UpdatesFromErpTopic = "UpdatesFromErp";

        private static readonly string ServiceUri =
            ServiceBusEnvironment.CreateServiceUri("sb", SbNamespace, "").ToString();

        private static void Main()
        {
            Console.WriteLine("Starting the ErpSubscriber...");
            Console.WriteLine();

            var subscriptionName = string.Format("Branch{0}", Guid.NewGuid());
            var tokenUri = string.Format("api/BranchMessaging/{0}/{1}", UpdatesFromErpTopic, subscriptionName);

            using (var httpClient = new HttpClient {BaseAddress = new Uri("http://branchazureservices.azurewebsites.net/")})
            {
                var getTokenResponse = httpClient.GetAsync(tokenUri).Result;
                getTokenResponse.EnsureSuccessStatusCode();
                var subscriptionToken =
                    getTokenResponse
                        .Content
                        .ReadAsStringAsync()
                        .Result
                        .Replace("\"", "");

                var factorySettings = new MessagingFactorySettings
                {
                    TransportType = TransportType.NetMessaging,
                    TokenProvider =
                        TokenProvider.CreateSharedAccessSignatureTokenProvider(subscriptionToken)
                };

                var factory = MessagingFactory.Create(ServiceUri, factorySettings);
                var subscriptionClient = factory.CreateSubscriptionClient(UpdatesFromErpTopic, subscriptionName);

                subscriptionClient.OnMessage(
                    brokeredMessage =>
                    {
                        var customer = brokeredMessage.GetBody<Customer>();
                        Console.WriteLine("ErpSubscriber received Customer {0}", customer.CustomerId);
                    });

                Console.WriteLine("Press any key to quit...");

                httpClient.DeleteAsync(tokenUri).Wait();
            }
        }
    }

}
