﻿using System;
using AzureHybrid.Models;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;

namespace BranchSubscriber
{
    internal class Program
    {
        private const string SbNamespace = "BaAzureHybrid";
        private const string UpdatesFromBranchesTopic = "UpdatesFromBranches";
        private const string ErpSubscription = "ErpSubscription";

        private const string SubscriptionPolicy = "SubscribeMessages";
        private const string SubscriptionPolicyKey = "<Your SubscriptionPolidyKey>";

        private static readonly string ServiceUri =
            ServiceBusEnvironment.CreateServiceUri("sb", SbNamespace, "").ToString();

        private static void Main()
        {
            Console.WriteLine("Starting the BranchSubscriber...");
            Console.WriteLine();

            var factorySettings = new MessagingFactorySettings
            {
                TransportType = TransportType.NetMessaging,
                TokenProvider =
                    TokenProvider.CreateSharedAccessSignatureTokenProvider(SubscriptionPolicy, SubscriptionPolicyKey)
            };

            var factory = MessagingFactory.Create(ServiceUri, factorySettings);
            var subscriptionClient = factory.CreateSubscriptionClient(UpdatesFromBranchesTopic, ErpSubscription);

            subscriptionClient.OnMessage(
                brokeredMessage =>
                {
                    var sale = brokeredMessage.GetBody<Sale>();
                    Console.WriteLine("BranchSubscriber received Sale of {0} from Customer {1}", sale.Quantity,
                        sale.CustomerId);
                });

            Console.WriteLine("Press any key to quit...");
            Console.ReadKey();
        }
    }
}
