﻿using System;
using AzureHybrid.Models;
using Microsoft.ServiceBus;
using Microsoft.ServiceBus.Messaging;

namespace ErpPublisher
{
    class Program
    {
        private const string SbNamespace = "BaAzureHybrid";
        private const string UpdatesFromErpTopic = "UpdatesFromErp";
        private const string PublishPolicy = "PublishMessages";
        private const string PublishPolicyKey = "<Your PublishPolidyKey>";

        private static readonly string ServiceUri = 
            ServiceBusEnvironment.CreateServiceUri("sb", SbNamespace, "").ToString();

        private static Customer CreateTestCustomer()
        {
            return new Customer
            {
                Name = "Titus",
                Company = "Titus Andronicus",
                Email = "Titus@Andronicus.com",
                Telephone = "111-222-3333"
            };
        }

        static void Main()
        {
            Console.WriteLine("Starting the ErpPublisher...");

            var factorySettings = new MessagingFactorySettings
            {
                TransportType = TransportType.NetMessaging,
                TokenProvider = TokenProvider.CreateSharedAccessSignatureTokenProvider(PublishPolicy, PublishPolicyKey)
            };

            var factory = MessagingFactory.Create(ServiceUri, factorySettings);
            var publisherClient = factory.CreateTopicClient(UpdatesFromErpTopic);

            var quitChar = ' ';

            Console.WriteLine("Press q to quit...");

            while (quitChar != 'q')
            {
                var customer = CreateTestCustomer();
                var message = new BrokeredMessage(customer);
                publisherClient.SendAsync(message).Wait();

                Console.WriteLine("Published Customer {0}", customer.CustomerId);

                quitChar = Console.ReadKey().KeyChar;
            }
        }
    }
}
